<?php declare(strict_types = 1);

namespace Drupal\Tests\debug_bar\Functional;

use Drupal\Tests\BrowserTestBase;
use Symfony\Component\Validator\Constraints\EqualTo;
use Symfony\Component\Validator\Constraints\Positive;
use Symfony\Component\Validator\Constraints\PositiveOrZero;
use Symfony\Component\Validator\Constraints\Regex;
use Trafaret\PhpUnit\TrafaretTrait;
use Trafaret\Trafaret;

/**
 * Debug bar test.
 *
 * @group debug_bar
 */
final class DebugBarTest extends BrowserTestBase {

  use TrafaretTrait;

  protected $defaultTheme = 'stark';
  protected static $modules = ['debug_bar'];

  /**
   * Test callback.
   */
  public function testAsUnprivilegedUser(): void {
    $user = $this->drupalCreateUser(['view debug bar']);
    $this->drupalLogin($user);

    $this->drupalGet('<front>');
    $debug_bar = $this->xpath('//div[@id = "debug-bar"]')[0];
    $expected_html = \file_get_contents(__DIR__ . '/unprivileged.html.trafaret');

    $this->assertStringByTrafaret(
      new Trafaret($expected_html, $this->getConstraints()),
      $debug_bar->getOuterHtml(),
    );
  }

  /**
   * Test callback.
   */
  public function testPrivilegedUser(): void {
    $user = $this->drupalCreateUser(
      ['view debug bar', 'administer site configuration', 'access site reports'],
    );
    $this->drupalLogin($user);

    $this->drupalGet('<front>');
    $debug_bar = $this->xpath('//div[@id = "debug-bar"]')[0];
    $expected_html = \file_get_contents(__DIR__ . '/privileged.html.trafaret');

    $this->assertStringByTrafaret(
      new Trafaret($expected_html, $this->getConstraints()),
      $debug_bar->getOuterHtml(),
    );
  }

  /**
   * Returns constraint list for a trafaret.
   */
  private function getConstraints(): array {
    return [
      'base_path' => new EqualTo(\base_path()),
      'image_path' => new EqualTo(\base_path() . \Drupal::service('extension.list.module')->getPath('debug_bar') . '/images/'),
      'user_name' => new EqualTo($this->loggedInUser->getDisplayName()),
      'user_id' => new EqualTo($this->loggedInUser->id()),
      'php_version' => new EqualTo(\PHP_VERSION),
      'current_path' => new EqualTo(\base_path() . 'user/' . $this->loggedInUser->id()),
      'last_run' => new PositiveOrZero(),
      // The branch can be a commit hash.
      'git_branch' => new Regex('/^.{2,40}$/'),
      'core_version' => new Regex('/^\d+\.\d+\.\d+/'),
      'execution_time' => new Positive(),
      'memory_usage' => new Positive(),
      'db_queries' => new Positive(),
    ];
  }

}
